@@ -8,4 +8,4 @@ FILE_SIZE_DOWNLOAD_LIMIT = 270000000
 DEFAULT_TIMEOUT = 10
 PARAM_SPLITTER = ';'
 
-DEFAULT_ERROR_STATUS_CODES = [500, 503]
+DEFAULT_ERROR_STATUS_CODES = [500, 502, 503]
